/*
 compile with:
 gcc `gtk-config --cflags` -o example example.c `gtk-config --libs` libeasygtk.a
*/

#include "easygtk.h"
#include "test.xpm"

GtkWidget *win, *win2, *box, *box2, *label, *entry, *entry2, *button, *list, *check, *nb, *fs, *br;

/* this is called when the user closes the window */
void cb_exit()
{
 e_show_message("EasyGTK Example", "You decided to quit!", exit);
}

/* cancel for the add command */
void cb_cancel()
{
 gtk_widget_destroy(win2);
}

/* this adds an item to the list */
void cb_ok()
{
 char *items[2] = { e_entry_get(entry), e_entry_get(entry2) };
 e_list_insert(list, items);
 gtk_widget_destroy(win2);
}

/* this prints the current selected item from the list */
void cb_print(GtkWidget *w, int c, int r)
{
 printf("%s\n", e_list_get());
}

/* this is the entry window to add items to the list */
void cb_add()
{
 win2 = e_window_create("Name Example", 400, 200, 150, 150, E_NO_FUNC);
 box = e_box_create(win2, E_VERTICAL, 2);
 box2 = e_box_create(box, E_HORIZONTAL, 0);
 label = e_label_create(box2, "Enter a name:");
 entry = e_entry_create(box2, "");
 box2 = e_box_create(box, E_HORIZONTAL, 0);
 label = e_label_create(box2, "Enter a number:");
 entry2 = e_entry_create(box2, "");
#if 0
 box2 = e_buttonbox_create(box, E_SPREAD, "Ok", "Cancel", cb_ok, cb_cancel);
#else
 box2 = e_buttonbox_unlim(box, E_SPREAD, 2, "Ok", cb_ok, "Cancel", cb_cancel);
#endif
}

/* this will remove the selected item from the list */
void cb_rem()
{
 e_list_remove(list);
}

/* this will return the current state of the checkbox */
void cb_print_check()
{
 printf("%d\n", e_checkbutton_get(check));
}

/* font selection was canceled */
void cb_font_cancel()
{
 gtk_widget_destroy(fs);
}

/* user clicked on the ok button */
void cb_font_ok()
{
 printf("%s\n", e_font_get(fs));
 gtk_widget_destroy(fs);
}

/* display a font selection window */
void cb_font()
{
 fs = e_font_create("Select a font", cb_font_ok, cb_font_cancel);
}

#ifdef USE_IMLIB
/* some imaging example, the showing of an image */
void cb_show_cancel()
{
 gtk_widget_destroy(br);
}

void cb_show_ok()
{
 e_image_window(e_browse_get(br));
 gtk_widget_destroy(br);
}

void cb_show()
{
 br = e_browse_create("Pick an image", cb_show_ok, cb_show_cancel);
}
#endif

#ifdef USE_MAGICK
void cb_mag_cancel()
{
 gtk_widget_destroy(br);
}
 
void cb_mag_ok()
{
 if(fork()) e_image_display(e_browse_get(br));
 gtk_widget_destroy(br);
}
   
void cb_mag()
{
 br = e_browse_create("Pick an image", cb_mag_ok, cb_mag_cancel);
}
#endif

void cb_clicked()
{
 printf("click at: %d %d\n", e_event_get_x(), e_event_get_y());
}

/* this will make the main window */
int main(int argc, char *argv[])
{
 char *rbitems[4] = { "Choice 1", "Choice 2", "Choice 3", "Choice 4" };
 gtk_init(&argc, &argv);
#ifdef USE_IMLIB
 gdk_init(&argc, &argv);
 gdk_imlib_init();
#endif
 printf("You are using EasyGTK version %s\n", E_VERSION);
 win = e_window_create("EasyGTK Example", 340, 450, 100, 100, exit);
 box = e_box_create(win, E_VERTICAL, 5);
 nb = e_notebook_create(box, E_TOP);
 box = e_notebook_new_page(nb, "List", "This is a list example");
 label = e_label_create(box, "This is an example list:");
 list = e_list_create(box, 2, cb_print);
 e_set_size(list, 200, 300);
 e_list_setup(list, 0, "Names", 100);
 e_list_setup(list, 1, "Numbers", 100);
 box2 = e_buttonbox_unlim(box, E_SPREAD, 3, "Add Item", cb_add, "Remove Item", cb_rem, "Quit", exit);
 box = e_notebook_new_page(nb, "Other", "These are other examples");
 check = e_checkbutton_create(box, "This is a test", TRUE, cb_print_check);
 e_radiobuttons_create(box, rbitems, 4);
 e_button_create(box, "Select a font", cb_font);
 e_button_create(box, "Leave EasyGTK Example", cb_exit);
 box = e_notebook_new_page(nb, "New", "Some newer stuff");
 label = e_pixmap_create(test_xpm, win->window);
 e_button_create_with_pixmap(box, label, E_NO_FUNC);
#ifdef USE_IMLIB
 e_button_create(box, "Show image with IMlib", cb_show);
#endif
#ifdef USE_MAGICK
 e_button_create(box, "Show image with ImageMagick", cb_mag);
#endif
 box = e_notebook_new_page(nb, "Events", "Events example");
 box2 = e_event_create(box, 200, 300, cb_clicked);
#ifdef USE_IMLIB
 e_image_scaled(box2, "test.xpm", 200, 300);
#endif
 gtk_main();
 return 0;
}

